@@ -225,67 +225,6 @@ void HeaderGenerator::writeFunction(QTextStream& s, const AbstractMetaFunction*
     }
 }
 
-void HeaderGenerator::writeTypeConverterDecl(QTextStream& s, const TypeEntry* type)
-{
-    s << "template<>" << endl;
-
-    const AbstractMetaClass* metaClass = classes().findClass(type->name());
-    bool isAbstractOrObjectType = (metaClass &&  metaClass->isAbstract()) || type->isObject();
-
-    AbstractMetaFunctionList implicitConvs;
-    foreach (AbstractMetaFunction* func, implicitConversions(type)) {
-        if (!func->isUserAdded())
-            implicitConvs << func;
-    }
-    bool isValueTypeWithImplConversions = type->isValue() && !implicitConvs.isEmpty();
-    bool hasCustomConversion = type->hasNativeConversionRule();
-    QString typeT = type->qualifiedCppName() + (isAbstractOrObjectType ? "*" : "");
-    QString typeName = type->qualifiedCppName();
-
-    if (avoidProtectedHack()) {
-        const AbstractMetaEnum* metaEnum = findAbstractMetaEnum(type);
-        if (metaEnum && metaEnum->isProtected()) {
-            typeT = protectedEnumSurrogateName(metaEnum);
-            typeName = typeT;
-        }
-    }
-
-    typeT.prepend("::");
-    typeName.prepend("::");
-
-    s << "struct Converter< " << typeT << " >";
-    if (!hasCustomConversion) {
-        if (type->isEnum())
-            s << " : EnumConverter";
-        else if (type->isFlags())
-            s << " : QFlagsConverter";
-        else if (isAbstractOrObjectType)
-            s << " : ObjectTypeConverter";
-        else
-            s << " : ValueTypeConverter";
-        s << "< " << typeName << " >";
-    }
-    s << endl << '{' << endl;
-    if (isValueTypeWithImplConversions || hasCustomConversion) {
-        s << INDENT << "static " << typeName << " toCpp(PyObject* pyobj);" << endl;
-        s << INDENT << "static bool isConvertible(PyObject* pyobj);" << endl;
-        if (hasCustomConversion) {
-            s << INDENT << "static bool checkType(PyObject* pyobj);" << endl;
-            s << INDENT << "static inline PyObject* toPython(void* cppObj) { return toPython(*reinterpret_cast< "
-              << typeName << (isAbstractOrObjectType ? "" : "*") << " >(cppObj)); }" << endl;
-            s << INDENT << "static PyObject* toPython(const " << typeName << "& cppObj);" << endl;
-        }
-    }
-    s << "};" << endl;
-
-    // write value-type like converter to object-types
-    if (isAbstractOrObjectType) {
-        s << endl << "template<>" << endl;
-        s << "struct Converter< " << typeName << " > : ObjectTypeReferenceConverter< " << typeName << " >" << endl << '{' << endl;
-        s << "};" << endl;
-    }
-}
-
 static void _writeTypeIndexDefineLine(QTextStream& s, const QString& variableName, int typeIndex)
 {
     s << "#define ";
@@ -336,12 +275,8 @@ void HeaderGenerator::finishGeneration()
     QSet<Include> includes;
     QString macros;
     QTextStream macrosStream(&macros);
-    QString convertersDecl;
-    QTextStream convDecl(&convertersDecl);
     QString sbkTypeFunctions;
     QTextStream typeFunctions(&sbkTypeFunctions);
-    QString converterImpl;
-    QTextStream convImpl(&converterImpl);
     QString protectedEnumSurrogates;
     QTextStream protEnumsSurrogates(&protectedEnumSurrogates);
 
@@ -360,16 +295,48 @@ void HeaderGenerator::finishGeneration()
     macrosStream << "SBK_"+moduleName()+"_IDX_COUNT";
     macrosStream.setFieldWidth(0);
     macrosStream << ' ' << getMaxTypeIndex() << endl << endl;
-    macrosStream << "// This variable stores all python types exported by this module" << endl;
+    macrosStream << "// This variable stores all Python types exported by this module." << endl;
     macrosStream << "extern PyTypeObject** " << cppApiVariableName() << ';' << endl << endl;
+    macrosStream << "// This variable stores all type converters exported by this module." << endl;
+    macrosStream << "extern SbkConverter** " << convertersVariableName() << ';' << endl << endl;;
+
+    // TODO-CONVERTER ------------------------------------------------------------------------------
+    // Using a counter would not do, a fix must be made to APIExtractor's getTypeIndex().
+    macrosStream << "// Converter indices" << endl;
+    QList<const PrimitiveTypeEntry*> primitives = primitiveTypes();
+    int pCount = 0;
+    foreach (const PrimitiveTypeEntry* ptype, primitives) {
+        if (!ptype->generateCode() || !isUserPrimitive(ptype))
+            continue;
+        _writeTypeIndexDefineLine(macrosStream, getTypeIndexVariableName(ptype), pCount);
+        pCount++;
+    }
+
+    foreach (const AbstractMetaType* container, instantiatedContainers()) {
+        //_writeTypeIndexDefineLine(macrosStream, getTypeIndexVariableName(container), pCount);
+        // DEBUG
+        QString variableName = getTypeIndexVariableName(container);
+        macrosStream << "#define ";
+        macrosStream.setFieldWidth(60);
+        macrosStream << variableName;
+        macrosStream.setFieldWidth(0);
+        macrosStream << ' ' << pCount << " // " << container->cppSignature() << endl;
+        // DEBUG
+        pCount++;
+    }
+
+    // Because on win32 the compiler will not accept a zero length array.
+    if (pCount == 0)
+        pCount++;
+    _writeTypeIndexDefineLine(macrosStream, QString("SBK_%1_CONVERTERS_IDX_COUNT").arg(moduleName()), pCount);
+    macrosStream << endl;
+    // TODO-CONVERTER ------------------------------------------------------------------------------
 
     macrosStream << "// Macros for type check" << endl;
     foreach (const AbstractMetaEnum* cppEnum, globalEnums) {
         if (cppEnum->isAnonymous() || cppEnum->isPrivate())
             continue;
         includes << cppEnum->typeEntry()->include();
-        writeTypeConverterDecl(convDecl, cppEnum->typeEntry());
-        convDecl << endl;
         writeProtectedEnumSurrogate(protEnumsSurrogates, cppEnum);
         writeSbkTypeFunction(typeFunctions, cppEnum);
     }
@@ -387,21 +354,12 @@ void HeaderGenerator::finishGeneration()
                 continue;
             EnumTypeEntry* enumType = cppEnum->typeEntry();
             includes << enumType->include();
-            writeTypeConverterDecl(convDecl, enumType);
-            FlagsTypeEntry* flagsEntry = enumType->flags();
-            if (flagsEntry)
-                writeTypeConverterDecl(convDecl, flagsEntry);
-            convDecl << endl;
             writeProtectedEnumSurrogate(protEnumsSurrogates, cppEnum);
             writeSbkTypeFunction(typeFunctions, cppEnum);
         }
 
-        if (!metaClass->isNamespace()) {
+        if (!metaClass->isNamespace())
             writeSbkTypeFunction(typeFunctions, metaClass);
-            writeTypeConverterDecl(convDecl, classType);
-            writeTypeConverterImpl(convImpl, classType);
-            convDecl << endl;
-        }
     }
 
     QString moduleHeaderFileName(outputDirectory()
@@ -469,42 +427,8 @@ void HeaderGenerator::finishGeneration()
     s << "// PyType functions, to get the PyObjectType for a type T\n";
     s << sbkTypeFunctions << endl;
 
-    if (usePySideExtensions()) {
-        foreach (AbstractMetaClass* metaClass, classes()) {
-            if (!metaClass->isQObject() || !metaClass->typeEntry()->generateCode())
-                continue;
-
-            s << "template<>" << endl;
-            s << "inline PyObject* createWrapper<" << metaClass->qualifiedCppName() << " >(const ";
-            s << metaClass->qualifiedCppName() << "* cppobj, bool hasOwnership, bool isExactType)" << endl;
-            s << '{' << endl;
-            s << INDENT << "const char* typeName = 0;" << endl;
-            s << INDENT << metaClass->qualifiedCppName() << "* value = const_cast<" << metaClass->qualifiedCppName() << "* >(cppobj);" << endl;
-            s << INDENT << "if (!isExactType)" << endl;
-            s << INDENT << INDENT << "typeName = typeid(*value).name();" << endl;
-            s << INDENT << "PyObject* pyObj = Shiboken::Object::newObject(reinterpret_cast<SbkObjectType*>(SbkType< ::" << metaClass->qualifiedCppName() << " >()),"
-            << "value, hasOwnership, isExactType, typeName);" << endl;
-            s << INDENT << "PySide::Signal::updateSourceObject(pyObj);" << endl;
-            s << INDENT << "return pyObj;" << endl;
-            s << '}' << endl;
-
-        }
-    }
-
-    s << "// Generated converters declarations ----------------------------------" << endl << endl;
-    s << convertersDecl;
     s << "} // namespace Shiboken" << endl << endl;
 
-    s << "// User defined converters --------------------------------------------" << endl;
-    foreach (TypeEntry* typeEntry, TypeDatabase::instance()->entries()) {
-        if (typeEntry->hasNativeConversionRule()) {
-            s << "// Conversion rule for: " << typeEntry->name() << endl;
-            s << typeEntry->conversionRule();
-        }
-    }
-    s << "// Generated converters implemantations -------------------------------" << endl << endl;
-    s << converterImpl << endl;
-
     s << "#endif // " << includeShield << endl << endl;
 }
 
@@ -541,108 +465,6 @@ void HeaderGenerator::writeSbkTypeFunction(QTextStream& s, const AbstractMetaCla
       <<  "{ return reinterpret_cast<PyTypeObject*>(" << cpythonTypeNameExt(cppClass->typeEntry()) << "); }\n";
 }
 
-void HeaderGenerator::writeTypeConverterImpl(QTextStream& s, const TypeEntry* type)
-{
-    if (type->hasNativeConversionRule())
-        return;
-
-    QString pyTypeName = cpythonTypeName(type);
-
-    AbstractMetaFunctionList implicitConvs;
-    foreach (AbstractMetaFunction* func, implicitConversions(type)) {
-        if (!func->isUserAdded())
-            implicitConvs << func;
-    }
-
-    bool hasImplicitConversions = !implicitConvs.isEmpty();
-
-    // A specialized Converter<T>::toCpp method is only need for
-    // classes with implicit conversions.
-    if (!hasImplicitConversions)
-        return;
-
-    const QString typeName = "::" + type->qualifiedCppName();
-
-    // Write Converter<T>::isConvertible
-    s << "inline bool Shiboken::Converter< " << typeName << " >::isConvertible(PyObject* pyobj)" << endl;
-    s << '{' << endl;
-
-    if (type->isValue()) {
-        s << INDENT << "if (ValueTypeConverter< " << typeName << " >::isConvertible(pyobj))" << endl;
-        Indentation indent(INDENT);
-        s << INDENT << "return true;" << endl;
-    }
-
-
-    s << INDENT << "SbkObjectType* shiboType = reinterpret_cast<SbkObjectType*>(SbkType< ";
-    s << typeName << " >());" << endl;
-    s << INDENT << "return ";
-    bool isFirst = true;
-    foreach (const AbstractMetaFunction* ctor, implicitConvs) {
-        Indentation indent(INDENT);
-        if (isFirst)
-            isFirst = false;
-        else
-            s << endl << INDENT << " || ";
-        if (ctor->isConversionOperator())
-            s << cpythonCheckFunction(ctor->ownerClass()->typeEntry());
-        else
-            s << cpythonCheckFunction(ctor->arguments().first()->type());
-        s << "(pyobj)";
-    }
-    s << endl;
-    {
-        Indentation indent(INDENT);
-        s << INDENT << " || (ObjectType::isExternalConvertible(shiboType, pyobj));" << endl;
-    }
-    s << '}' << endl << endl;
-
-    // Write Converter<T>::toCpp function
-    s << "inline " << typeName << " Shiboken::Converter< " << typeName << " >::toCpp(PyObject* pyobj)" << endl;
-    s << '{' << endl;
-
-    s << INDENT << "if (PyObject_TypeCheck(pyobj, SbkType< " << typeName << " >()))" << endl;
-    {
-        Indentation indent(INDENT);
-        s << INDENT << "return *" << cpythonWrapperCPtr(type, "pyobj") << ';' << endl;
-    }
-
-    foreach (const AbstractMetaFunction* ctor, implicitConvs) {
-        if (ctor->isModifiedRemoved())
-            continue;
-
-        s << INDENT << "else ";
-
-        QString typeCheck;
-        QString toCppConv;
-        QTextStream tcc(&toCppConv);
-        if (ctor->isConversionOperator()) {
-            const AbstractMetaClass* metaClass = ctor->ownerClass();
-            typeCheck = cpythonCheckFunction(metaClass->typeEntry());
-            writeToCppConversion(tcc, metaClass, "pyobj");
-        } else {
-            const AbstractMetaType* argType = ctor->arguments().first()->type();
-            typeCheck = cpythonCheckFunction(argType);
-            writeToCppConversion(tcc, argType, 0, "pyobj");
-        }
-
-        s << "if (" << typeCheck << "(pyobj))" << endl;
-        {
-            Indentation indent(INDENT);
-            s << INDENT << "return " << type->name() << '(' << toCppConv << ");" << endl;
-        }
-    }
-
-    {
-        s << INDENT << "else" << endl;
-        {
-            Indentation indent(INDENT);
-            s << INDENT << "return Shiboken::ValueTypeConverter< " << typeName << " >::toCpp(pyobj);" << endl;
-        }
-    }
-    s << '}' << endl << endl;
-}
-
 void HeaderGenerator::writeInheritedOverloads(QTextStream& s)
 {
     foreach (const AbstractMetaFunction* func, m_inheritedOverloads) {
@@ -666,4 +488,3 @@ void HeaderGenerator::writeInheritedOverloads(QTextStream& s)
         s << "; }" << endl;
     }
 }
-
