@@ -40,6 +40,13 @@
 #define END_ALLOW_THREADS         "PyEval_RestoreThread(_save); // Py_END_ALLOW_THREADS"
 #define MIN_CTOR_ERROR_MSG        "Could not find a minimal constructor for type '%1'. "\
                                   "This will result in a compilation error."
+#define PYTHON_TO_CPP_VAR         "pythonToCpp"
+
+#define CHECKTYPE_REGEX           "%CHECKTYPE\\[([^\\[]*)\\]\\("
+#define ISCONVERTIBLE_REGEX       "%ISCONVERTIBLE\\[([^\\[]*)\\]\\("
+#define CONVERTTOPYTHON_REGEX     "%CONVERTTOPYTHON\\[([^\\[]*)\\]\\("
+#define CONVERTTOCPP_REGEX        "(\\*?%?[a-zA-Z_][\\w\\.]*(?:\\[[^\\[^<^>]+\\])*)"\
+                                  "(?:\\s+)=(?:\\s+)%CONVERTTOCPP\\[([^\\[]*)\\]\\("
 
 #include <generator.h>
 #include <QtCore/QTextStream>
@@ -241,17 +248,13 @@ public:
 
     const AbstractMetaClass* getMultipleInheritingClass(const AbstractMetaClass* metaClass);
 
-    void writeBaseConversion(QTextStream& s, const AbstractMetaType* type,
-                             const AbstractMetaClass* context = 0, Options options = NoOption);
-    /// Simpler version of writeBaseConversion, uses only the base name of the type.
-    void writeBaseConversion(QTextStream& s, const TypeEntry* type);
     void writeToPythonConversion(QTextStream& s, const AbstractMetaType* type,
                                  const AbstractMetaClass* context, const QString& argumentName);
-    void writeToCppConversion(QTextStream& s, const AbstractMetaType* type, const AbstractMetaClass* context, const QString& argumentName);
-    void writeToCppConversion(QTextStream& s, const AbstractMetaClass* metaClass, const QString& argumentName);
+    void writeToCppConversion(QTextStream& s, const AbstractMetaType* type, const AbstractMetaClass* context, const QString& inArgName, const QString& outArgName);
+    void writeToCppConversion(QTextStream& s, const AbstractMetaClass* metaClass, const QString& inArgName, const QString& outArgName);
 
     /// Returns true if the argument is a pointer that rejects NULL values.
-    static bool shouldRejectNullPointerArgument(const AbstractMetaFunction* func, int argIndex);
+    bool shouldRejectNullPointerArgument(const AbstractMetaFunction* func, int argIndex);
 
     /// Verifies if the class should have a C++ wrapper generated for it, instead of only a Python wrapper.
     bool shouldGenerateCppWrapper(const AbstractMetaClass* metaClass) const;
@@ -278,12 +281,15 @@ public:
 
     static QString cpythonOperatorFunctionName(const AbstractMetaFunction* func);
 
+    static QString fixedCppTypeName(const CustomConversion::TargetToNativeConversion* toNative);
+    static QString fixedCppTypeName(const AbstractMetaType* type);
+    static QString fixedCppTypeName(const TypeEntry* type, QString typeName = QString());
+
     static bool isNumber(QString cpythonApiName);
     static bool isNumber(const TypeEntry* type);
     static bool isNumber(const AbstractMetaType* type);
     static bool isPyInt(const TypeEntry* type);
     static bool isPyInt(const AbstractMetaType* type);
-    static bool isCString(const AbstractMetaType* type);
     static bool isPairContainer(const AbstractMetaType* type);
 
     /**
@@ -300,6 +306,27 @@ public:
      */
     static bool isPointerToWrapperType(const AbstractMetaType* type);
 
+    /**
+     *  Returns true if \p type is an Object Type used as a value.
+     */
+    static bool isObjectTypeUsedAsValueType(const AbstractMetaType* type);
+
+    static bool isValueTypeWithCopyConstructorOnly(const AbstractMetaClass* metaClass);
+    bool isValueTypeWithCopyConstructorOnly(const TypeEntry* type) const;
+    bool isValueTypeWithCopyConstructorOnly(const AbstractMetaType* type) const;
+
+    /// Returns true if the type is a primitive but not a C++ primitive.
+    static bool isUserPrimitive(const TypeEntry* type);
+    static bool isUserPrimitive(const AbstractMetaType* type);
+
+    /// Returns true if the type is a C++ primitive, a void*, a const char*, or a std::string.
+    static bool isCppPrimitive(const TypeEntry* type);
+    static bool isCppPrimitive(const AbstractMetaType* type);
+
+    /// Returns true if the type is a C++ integral primitive, i.e. bool, char, int, long, and their unsigned counterparts.
+    static bool isCppIntegralPrimitive(const TypeEntry* type);
+    static bool isCppIntegralPrimitive(const AbstractMetaType* type);
+
     /// Checks if an argument type should be dereferenced by the Python method wrapper before calling the C++ method.
     static bool shouldDereferenceArgumentPointer(const AbstractMetaArgument* arg);
     /// Checks if a meta type should be dereferenced by the Python method wrapper passing it to C++.
@@ -307,6 +334,9 @@ public:
 
     static bool visibilityModifiedToPrivate(const AbstractMetaFunction* func);
 
+    QString converterObject(const AbstractMetaType* type);
+    QString converterObject(const TypeEntry* type);
+
     QString cpythonBaseName(const AbstractMetaClass* metaClass);
     QString cpythonBaseName(const TypeEntry* type);
     QString cpythonBaseName(const AbstractMetaType* type);
@@ -394,6 +424,7 @@ public:
     /// Returns true if the generated code should use the "#define protected public" hack.
     bool avoidProtectedHack() const;
     QString cppApiVariableName(const QString& moduleName = QString()) const;
+    QString convertersVariableName(const QString& moduleName = QString()) const;
     /**
      *  Returns the type index variable name for a given class. If \p alternativeTemplateName is true
      *  and the class is a typedef for a template class instantiation, it will return an alternative name
@@ -404,11 +435,6 @@ public:
     QString getTypeIndexVariableName(const TypeEntry* type);
     QString getTypeIndexVariableName(const AbstractMetaType* type);
 
-    /// Returns the proper full name for \p type.
-    QString getFullTypeName(const TypeEntry* type);
-    QString getFullTypeName(const AbstractMetaType* type);
-    QString getFullTypeName(const AbstractMetaClass* metaClass);
-
     /// Returns true if the user don't want verbose error messages on the generated bindings.
     bool verboseErrorMessagesDisabled() const;
 
@@ -420,6 +446,14 @@ public:
      */
     AbstractMetaType* buildAbstractMetaTypeFromString(QString typeSignature);
 
+    /// Creates an AbstractMetaType object from a TypeEntry.
+    AbstractMetaType* buildAbstractMetaTypeFromTypeEntry(const TypeEntry* typeEntry);
+    /// Creates an AbstractMetaType object from an AbstractMetaClass.
+    AbstractMetaType* buildAbstractMetaTypeFromAbstractMetaClass(const AbstractMetaClass* metaClass);
+
+    void writeMinimalConstructorExpression(QTextStream& s, const AbstractMetaType* type, const QString& defaultCtor = QString());
+    void writeMinimalConstructorExpression(QTextStream& s, const TypeEntry* type, const QString& defaultCtor = QString());
+
     /**
      *  Helper function to return the flags to be used by a meta type when
      * it needs to write some converter code.
@@ -432,6 +466,7 @@ public:
     static QString getDefaultValue(const AbstractMetaFunction* func, const AbstractMetaArgument* arg);
 protected:
     bool doSetup(const QMap<QString, QString>& args);
+    void collectContainerTypesFromConverterMacros(const QString& code, bool toPythonMacro);
     // verify whether the class is copyable
     bool isCopyable(const AbstractMetaClass* metaClass);
 
@@ -465,6 +500,9 @@ protected:
     /// Returns all extended conversions for the current module.
     ExtendedConverterData getExtendedConverters() const;
 
+    /// Returns a list of converters for the non wrapper types of the current module.
+    QList<const CustomConversion*> getPrimitiveCustomConversions();
+
     /// Returns true if the Python wrapper for the received OverloadData must accept a list of arguments.
     static bool pythonFunctionWrapperUsesListOfArguments(const OverloadData& overloadData);
 
