@@ -22,6 +22,7 @@
 
 #include "basewrapper.h"
 #include "basewrapper_p.h"
+#include "sbkconverter.h"
 #include "sbkenum.h"
 #include "autodecref.h"
 #include "typeresolver.h"
@@ -29,6 +30,8 @@
 #include <string>
 #include <cstring>
 #include <cstddef>
+#include <set>
+#include <sstream>
 #include <algorithm>
 #include "threadstatesaver.h"
 
@@ -210,7 +213,7 @@ void SbkDeallocWrapper(PyObject* pyObj)
     if (sbkObj->d->hasOwnership && sbkObj->d->validCppObject) {
         SbkObjectType* sbkType = reinterpret_cast<SbkObjectType*>(pyObj->ob_type);
         if (sbkType->d->is_multicpp) {
-            Shiboken::DtorCallerVisitor visitor(sbkObj);
+            Shiboken::DeallocVisitor visitor(sbkObj);
             Shiboken::walkThroughClassHierarchy(pyObj->ob_type, &visitor);
         } else {
             void* cptr = sbkObj->d->cptr[0];
@@ -248,6 +251,8 @@ void SbkObjectTypeDealloc(PyObject* pyObj)
         }
         free(sbkType->d->original_name);
         sbkType->d->original_name = 0;
+        if (!Shiboken::ObjectType::isUserType(reinterpret_cast<PyTypeObject*>(sbkType)))
+            Shiboken::Conversions::deleteConverter(sbkType->d->converter);
         delete sbkType->d;
         sbkType->d = 0;
     }
@@ -295,20 +300,18 @@ PyObject* SbkObjectTypeTpNew(PyTypeObject* metatype, PyObject* args, PyObject* k
         d->mi_offsets = parentType->mi_offsets;
         d->mi_init = parentType->mi_init;
         d->mi_specialcast = parentType->mi_specialcast;
-        d->ext_isconvertible = parentType->ext_isconvertible;
-        d->ext_tocpp = parentType->ext_tocpp;
         d->type_discovery = parentType->type_discovery;
         d->cpp_dtor = parentType->cpp_dtor;
         d->is_multicpp = 0;
+        d->converter = parentType->converter;
     } else {
         d->mi_offsets = 0;
         d->mi_init = 0;
         d->mi_specialcast = 0;
-        d->ext_isconvertible = 0;
-        d->ext_tocpp = 0;
         d->type_discovery = 0;
         d->cpp_dtor = 0;
         d->is_multicpp = 1;
+        d->converter = 0;
     }
     if (bases.size() == 1)
         d->original_name = strdup(bases.front()->d->original_name);
@@ -444,8 +447,6 @@ void DtorCallerVisitor::visit(SbkObjectType* node)
 
 void DtorCallerVisitor::done()
 {
-    Shiboken::Object::deallocData(m_pyObj, true);
-
     std::list<std::pair<void*, SbkObjectType*> >::const_iterator it = m_ptrs.begin();
     for (; it != m_ptrs.end(); ++it) {
         Shiboken::ThreadStateSaver threadSaver;
@@ -454,7 +455,14 @@ void DtorCallerVisitor::done()
     }
 }
 
+void DeallocVisitor::done()
+{
+    Shiboken::Object::deallocData(m_pyObj, true);
+    DtorCallerVisitor::done();
+}
+
 namespace Module { void init(); }
+namespace Conversions { void init(); }
 
 void init()
 {
@@ -463,6 +471,7 @@ void init()
         return;
 
     Module::init();
+    Conversions::init();
 
     initTypeResolver();
     PyEval_InitThreads();
@@ -590,30 +599,13 @@ bool canCallConstructor(PyTypeObject* myType, PyTypeObject* ctorType)
     return true;
 }
 
-bool hasExternalCppConversions(SbkObjectType* self)
-{
-    return self->d->ext_tocpp;
-}
 
-void* callExternalCppConversion(SbkObjectType* self, PyObject* obj)
-{
-    return self->d->ext_tocpp(obj);
-}
+bool hasExternalCppConversions(SbkObjectType*) { return false; }                    // DEPRECATED.
+bool isExternalConvertible(SbkObjectType* self, PyObject* obj) { return false; }    // DEPRECATED.
+void setExternalCppConversionFunction(SbkObjectType*, ExtendedToCppFunc) {}         // DEPRECATED.
+void setExternalIsConvertibleFunction(SbkObjectType*, ExtendedIsConvertibleFunc) {} // DEPRECATED.
+void* callExternalCppConversion(SbkObjectType*, PyObject*) { return 0; }            // DEPRECATED.
 
-void setExternalCppConversionFunction(SbkObjectType* self, ExtendedToCppFunc func)
-{
-    self->d->ext_tocpp = func;
-}
-
-void setExternalIsConvertibleFunction(SbkObjectType* self, ExtendedIsConvertibleFunc func)
-{
-    self->d->ext_isconvertible = func;
-}
-
-bool isExternalConvertible(SbkObjectType* self, PyObject* obj)
-{
-    return self->d->ext_isconvertible && self->d->ext_isconvertible(obj);
-}
 
 bool hasCast(SbkObjectType* type)
 {
@@ -785,6 +777,27 @@ bool hasCppWrapper(SbkObject* pyObj)
     return pyObj->d->containsCppWrapper;
 }
 
+bool wasCreatedByPython(SbkObject* pyObj)
+{
+    return pyObj->d->cppObjectCreated;
+}
+
+void callCppDestructors(SbkObject* pyObj)
+{
+    SbkObjectType* sbkType = reinterpret_cast<SbkObjectType*>(Py_TYPE(pyObj));
+    if (sbkType->d->is_multicpp) {
+        Shiboken::DtorCallerVisitor visitor(pyObj);
+        Shiboken::walkThroughClassHierarchy(Py_TYPE(pyObj), &visitor);
+    } else {
+        Shiboken::ThreadStateSaver threadSaver;
+        threadSaver.save();
+        sbkType->d->cpp_dtor(pyObj->d->cptr[0]);
+    }
+    delete[] pyObj->d->cptr;
+    pyObj->d->cptr = 0;
+    invalidate(pyObj);
+}
+
 bool hasOwnership(SbkObject* pyObj)
 {
     return pyObj->d->hasOwnership;
@@ -819,7 +832,7 @@ void releaseOwnership(SbkObject* self)
 {
     // skip if the ownership have already moved to c++
     SbkObjectType* selfType = reinterpret_cast<SbkObjectType*>(Py_TYPE(self));
-    if (!self->d->hasOwnership || selfType->d->type_behaviour == BEHAVIOUR_VALUETYPE)
+    if (!self->d->hasOwnership || Shiboken::Conversions::pythonTypeIsValueType(selfType->d->converter))
         return;
 
     // remove object ownership
@@ -935,6 +948,16 @@ void* cppPointer(SbkObject* pyObj, PyTypeObject* desiredType)
     return 0;
 }
 
+std::vector<void*> cppPointers(SbkObject* pyObj)
+{
+    int n = getNumberOfCppBaseClasses(Py_TYPE(pyObj));
+    std::vector<void*> ptrs(n);
+    for (int i = 0; i < n; ++i)
+        ptrs[i] = pyObj->d->cptr[i];
+    return ptrs;
+}
+
+
 bool setCppPointer(SbkObject* sbkObj, PyTypeObject* desiredType, void* cptr)
 {
     int idx = 0;
@@ -1011,13 +1034,13 @@ PyObject* newObject(SbkObjectType* instanceType,
 {
     // Try to find the exact type of cptr.
     if (!isExactType) {
-        TypeResolver* tr = 0;
+        PyTypeObject* exactType = 0;
         if (typeName) {
-            tr = TypeResolver::get(typeName);
-            if (tr)
-                instanceType = reinterpret_cast<SbkObjectType*>(tr->pythonType());
+            exactType = Shiboken::Conversions::getPythonTypeObject(typeName);
+            if (exactType)
+                instanceType = reinterpret_cast<SbkObjectType*>(exactType);
         }
-        if (!tr)
+        if (!exactType)
             instanceType = BindingManager::instance().resolveType(&cptr, instanceType);
     }
 
@@ -1282,6 +1305,64 @@ void clearReferences(SbkObject* self)
     self->d->referredObjects->clear();
 }
 
+std::string info(SbkObject* self)
+{
+    std::ostringstream s;
+    std::list<SbkObjectType*> bases;
+
+    if (ObjectType::isUserType(Py_TYPE(self)))
+        bases = getCppBaseClasses(Py_TYPE(self));
+    else
+        bases.push_back(reinterpret_cast<SbkObjectType*>(Py_TYPE(self)));
+
+    s << "C++ address....... ";
+    std::list<SbkObjectType*>::const_iterator it = bases.begin();
+    for (int i = 0; it != bases.end(); ++it, ++i)
+        s << ((PyTypeObject*)*it)->tp_name << "/" << self->d->cptr[i] << ' ';
+    s << "\n";
+
+    s << "hasOwnership...... " << bool(self->d->hasOwnership) << "\n"
+         "containsCppWrapper " << self->d->containsCppWrapper << "\n"
+         "validCppObject.... " << self->d->validCppObject << "\n"
+         "wasCreatedByPython " << self->d->cppObjectCreated << "\n";
+
+
+    if (self->d->parentInfo && self->d->parentInfo->parent) {
+        s << "parent............ ";
+        Shiboken::AutoDecRef parent(PyObject_Str((PyObject*)self->d->parentInfo->parent));
+        s << String::toCString(parent) << "\n";
+    }
+
+    if (self->d->parentInfo && self->d->parentInfo->children.size()) {
+        s << "children.......... ";
+        ChildrenList& children = self->d->parentInfo->children;
+        for (ChildrenList::const_iterator it = children.begin(); it != children.end(); ++it) {
+            Shiboken::AutoDecRef child(PyObject_Str((PyObject*)*it));
+            s << String::toCString(child) << ' ';
+        }
+        s << '\n';
+    }
+
+    if (self->d->referredObjects && self->d->referredObjects->size()) {
+        Shiboken::RefCountMap& map = *self->d->referredObjects;
+        s << "referred objects.. ";
+        Shiboken::RefCountMap::const_iterator it = map.begin();
+        for (; it != map.end(); ++it) {
+            if (it != map.begin())
+                s << "                   ";
+            s << '"' << it->first << "\" => ";
+            std::list<PyObject*>::const_iterator j = it->second.begin();
+            for (; j != it->second.end(); ++j) {
+                Shiboken::AutoDecRef obj(PyObject_Str(*j));
+                s << String::toCString(obj) << ' ';
+            }
+            s << ' ';
+        }
+        s << '\n';
+    }
+    return s.str();
+}
+
 } // namespace Object
 
 } // namespace Shiboken
