@@ -42,6 +42,11 @@
         }
 #endif
 
+extern "C"
+{
+struct SbkConverter;
+}
+
 namespace Shiboken {
 namespace Module {
 
@@ -74,6 +79,20 @@ LIBSHIBOKEN_API void registerTypes(PyObject* module, PyTypeObject** types);
  */
 LIBSHIBOKEN_API PyTypeObject** getTypes(PyObject* module);
 
+/**
+ *  Registers the list of converters created by \p module for non-wrapper types.
+ *  \param module       Module where the converters were created.
+ *  \param converters   Array of SbkConverter* objects representing the converters created on \p module.
+ */
+LIBSHIBOKEN_API void registerTypeConverters(PyObject* module, SbkConverter** converters);
+
+/**
+ *  Retrieves the array of converters.
+ *  \param module   Module where the converters were created.
+ *  \returns        A pointer to the SbkConverter* array of converters.
+ */
+LIBSHIBOKEN_API SbkConverter** getTypeConverters(PyObject* module);
+
 } } // namespace Shiboken::Module
 
 #endif // SBK_MODULE_H
